/////////////////////////////////////////
// Title: Terrain multitexture material
// Conitec / jcl 2004
/////////////////////////////////////////
// Blends 3 textures together, according to
// the red and green channel in the first skin.
// The blue channel can be used for a shadow map.

// Skin1 - Terrain RGB blend & shadow mask
// Skin2 - Base terrain texture
// Skin3 - Red channel masked tiled texture
// Skin4 - Green channel masked tiled texture
// Skin5 - Replacement base texture for non-shader hardware
// Skin6 - Replacement detail texture for non-shader hardware

// Skill41 - Skin2 scale, float(1..50)
// Skill42 - Skin3 scale, float(1..50)
// Skill43 - Skin4 scale, float(1..50)
// Skill44 - ambient, float(-1..1)

// Based on concepts by Thomas 'ventilator' Oppl
// and Eric 'Steempipe' Hendrickson-Lambert

// Requires A6.31, VS 1.1, PS 1.1

//todo: pass real sun color
//todo: blend 2 more textures on blue and alpha in a second pass

function MAT_Tile_Init()
{
// Adjust the texture transform to do some tiling of the textures

MTL.MATRIX11 = FLOAT(32);	// default u scale 
MTL.MATRIX22 = FLOAT(32);	// default v scale

}

MATERIAL MAT_Tile_Only
{

event = MAT_Tile_Init;

	effect = "

		matrix matMtl;

		texture entSkin2;	// Tileable Texutre

technique ffp_terrain_tex
{
		pass P0
		//////////////////////////////////////////////////////
		//// Lets lay the texture down. no alpha work
		//////////////////////////////////////////////////////
		{
			AlphaBlendEnable = False;
			AlphaTestEnable  = False;

			Lighting = True;

			zEnable = True;
			zWriteEnable = True;
			
			////////////////////////////////////////
			// Stage 0
			///////////////////////////////////////

			Texture[0] = <entSkin2>;	

			magFilter[0] = Linear;
			minFilter[0] = Linear;
			mipFilter[0] = Linear;
			
			TextureTransformFlags[0] = Count2;
			TextureTransform[0] = <matMtl>;
			texcoordindex[0] = 0;

			ColorArg1[0] = Texture; 
			ColorOp[0] = SelectArg1;
			ColorArg2[0] = Diffuse;
			AlphaOp[0] = Disable;
			
			//////////////////////////////////////
			// Stage 1
			//////////////////////////////////////

			ColorOp[1] = Disable;
			AlphaOp[1] = Disable;
           
		}

		} // End of Technique

   "; // end of the effect string

} // End of Material 

MATERIAL MAT_Terrain_Blendmap
{
	effect = "

	float4x4 matWorld;
	float4x4 matWorldInv;
	float4x4 matWorldView;
	float4x4 matWorldViewProj;

	float4 vecSunDir;
	float4 vecSunDiffuse = float4(200.f/255.f, 200.f/255.f, 200.f/255.f, 1.f);
	float4 vecFog;
	float4 vecLight;

	Texture entSkin1; // Red/green for blending, blue for shadow
	Texture entSkin2; // Basic tiled terrain texture
	Texture entSkin3; // Red masked tiled texture
	Texture entSkin4; // Green masked tiled texture

	float4 vecSkill41;

	float4 vecLightPos[8];	 // preset this with light positions (xyz) and ranges (w)
	float4 vecLightColor[8]; // preset this with light colors
	float3 vecFalloff = float3(0.f, 0.f, 1.5f);


	sampler sMaskTex = sampler_state
	{
		Texture = <entSkin1>;
		MipFilter = LINEAR;
		MinFilter = LINEAR;
		MagFilter = LINEAR;
		AddressU = Wrap;
		Addressv = Wrap;
	};

	sampler sBaseTex = sampler_state
	{
		Texture = <entSkin2>;
		MipFilter = LINEAR;
		MinFilter = LINEAR;
		MagFilter = LINEAR;
		AddressU = Wrap;
		Addressv = Wrap;
	};

	sampler sRedTex = sampler_state
	{
		Texture = <entSkin3>;
		MipFilter = LINEAR;
		MinFilter = LINEAR;
		MagFilter = LINEAR;
		AddressU = Wrap;
		Addressv = Wrap;
	};

	sampler sGreenTex = sampler_state
	{
		Texture = <entSkin4>;
		MipFilter = LINEAR;
		MinFilter = LINEAR;
		MagFilter = LINEAR;
		AddressU = Wrap;
		Addressv = Wrap;
	};

//////////////////////////////////////////////////////////////////////
// return the sun light on the surface
	float4 DoSunLight(float3 N)
	{
// modulate the sunlight by the surface angle
		return vecSunDiffuse * dot(N,-vecSunDir);
	}

// return the dynamic light on the surface
	float4 DoPointLight(float3 P, float3 N, int i)
	{
// calculate the light ray pointing from the light to the surface
		float3 D = (float3)vecLightPos[i]-P;
// calculate the angle between surface and light ray
		float NdotL = dot(N,normalize(D));
// modulate the light by the surface angle
		float4 Color = vecLightColor[i] * NdotL;

// calculate the light attenuation factor, DX uses a really strange formula here
		float fac = 0.f;
		if (NdotL >= 0.f && vecLightPos[i].w > 0.f)
		{
// get the distance factor
			float LD = length(D)/vecLightPos[i].w;
			if (LD < 1.f)
				fac = 1.f - LD;
		}
		return Color * fac;
	}


	float DoFog(float4 Pos)
	{
// convert the vector position to view space to get it's depth (.z)
		float3 P = mul(Pos,matWorldView);
// apply the linear fog formula
 		return saturate((vecFog.y-P.z) * vecFog.z);
	}

//////////////////////////////////////////////////////////////////////
	struct TMULTI_VS_OUT // Output to the pixelshader fragment
	{
		float4 Pos : POSITION;
		float4 Color: COLOR0;
		float  Fog: FOG;
		float2 MaskCoord  : TEXCOORD0;
		float2 BaseCoord  : TEXCOORD1;
		float2 RedCoord   : TEXCOORD2;
		float2 GreenCoord : TEXCOORD3;
	};


	TMULTI_VS_OUT TMulti_VS(
		float4 inPos : POSITION,
		float3 inNormal : NORMAL,
		float2 inTexCoord0 : TEXCOORD0)
	{
		TMULTI_VS_OUT Out;

// transform the vector position to screen coordinates
		Out.Pos = mul(inPos,matWorldViewProj);

// rotate and normalize the normal
		float3 N = normalize(mul(inNormal,matWorldInv));
		//float3 N = normalize(inNormal);
		float3 P = mul(inPos,matWorld);
// Add ambient and sun light
		Out.Color = vecSkill41.w + DoSunLight(N);
// Add 6 dynamic lights (maximum for vs 1.1)
		for (int i=0; i<6; i++)
			Out.Color += DoPointLight(P,N,i);
// Add fog
		Out.Fog = DoFog(inPos);

// scale the texture coordinates for the masked textures
		Out.MaskCoord  = inTexCoord0.xy;
		Out.BaseCoord  = inTexCoord0.xy * vecSkill41.x;
		Out.RedCoord   = inTexCoord0.xy * vecSkill41.y;
		Out.GreenCoord = inTexCoord0.xy * vecSkill41.z;
		return Out;
	}


	float4 TMulti_PS(TMULTI_VS_OUT In): COLOR
	{
// retrieve the pixels for the textures and the masks
		float4 MaskColor = tex2D(sMaskTex,In.MaskCoord);
		float4 BaseColor = tex2D(sBaseTex,In.BaseCoord);
		float4 RedColor  = tex2D(sRedTex,In.RedCoord);

// blend the red and green textures over the base texture
		float4 GreenColor = tex2D(sGreenTex,In.GreenCoord);
		float4 BaseRed = lerp(BaseColor,RedColor,MaskColor.r);
		float4 FinalColor = lerp(BaseRed,GreenColor,MaskColor.g);

// Add the vertex light or shadow map, plus the entity ambient
		FinalColor *= In.Color;
		FinalColor *= MaskColor.a;
//		FinalColor *= MaskColor.b + vecSkill41.w;
		FinalColor.a = 1.0f;	// prevent transparency
		return FinalColor;

	}


	technique tmulti3
	{
		pass one
		{
			sampler[0] = (sMaskTex);
			sampler[1] = (sBaseTex);
			sampler[2] = (sRedTex);
			sampler[3] = (sGreenTex);

			VertexShader = compile vs_1_1 TMulti_VS();
			PixelShader = compile ps_1_1 TMulti_PS();

			AlphaBlendEnable = False;
			AlphaTestEnable  = False;

			zEnable = True;
			zWriteEnable = True;

		}
	}
";
}

// C-Script stuff /////////////////////////////////////////////////////

// uses: ScaleSkin2 ScaleSkin3 ScaleSkin4
ACTION ffp_TerrainTex
{
	MY._charakter = 19;

	IF (d3d_shaderversion < 1111) {
		MY.MATERIAL = MAT_Tile_Only;
	}
		MY.MATERIAL = MAT_Terrain_Blendmap;
		MY.Skill41 = float(32);
		MY.Skill42 = float(32);
		MY.Skill43 = float(32);
		MY.Skill44 = float(MY.AMBIENT / 100);
	}

}